@@ -7,30 +7,26 @@
 #include "almalloc.h"
 #include "bufferline.h"
 #include "filters/splitter.h"
+#include "flexarray.h"
 
 
 struct FrontStablizer {
-    static constexpr size_t DelayLength{256u};
+    explicit FrontStablizer(size_t numchans) : ChannelFilters{numchans} { }
 
-    FrontStablizer(size_t numchans) : DelayBuf{numchans} { }
-
-    alignas(16) std::array<float,BufferLineSize + DelayLength> Side{};
-    alignas(16) std::array<float,BufferLineSize + DelayLength> MidDirect{};
-    alignas(16) std::array<float,DelayLength> MidDelay{};
-
-    alignas(16) std::array<float,BufferLineSize + DelayLength> TempBuf{};
+    alignas(16) std::array<float,BufferLineSize> MidDirect{};
+    alignas(16) std::array<float,BufferLineSize> Side{};
+    alignas(16) std::array<float,BufferLineSize> Temp{};
 
     BandSplitter MidFilter;
     alignas(16) FloatBufferLine MidLF{};
     alignas(16) FloatBufferLine MidHF{};
 
-    using DelayLine = std::array<float,DelayLength>;
-    al::FlexArray<DelayLine,16> DelayBuf;
+    al::FlexArray<BandSplitter,16> ChannelFilters;
 
     static std::unique_ptr<FrontStablizer> Create(size_t numchans)
     { return std::unique_ptr<FrontStablizer>{new(FamCount(numchans)) FrontStablizer{numchans}}; }
 
-    DEF_FAM_NEWDEL(FrontStablizer, DelayBuf)
+    DEF_FAM_NEWDEL(FrontStablizer, ChannelFilters)
 };
 
 #endif /* CORE_FRONT_STABLIZER_H */
